package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.sop.business.bean.result.SysMessageCountResult;
import com.qmth.sop.business.bean.result.SysMessageResult;
import com.qmth.sop.business.entity.SysMessage;
import com.qmth.sop.business.entity.SysNotice;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBService;
import com.qmth.sop.business.mapper.SysMessageMapper;
import com.qmth.sop.business.service.SysMessageService;
import com.qmth.sop.business.service.SysUserService;
import com.qmth.sop.business.service.TBServiceService;
import com.qmth.sop.business.service.TBUserArchivesSupplierService;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.FieldUniqueEnum;
import com.qmth.sop.common.enums.MessageTypeEnum;
import com.qmth.sop.common.enums.NoticeTypeEnum;
import com.qmth.sop.common.util.ResultUtil;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 系统消息 服务实现.
 *
 * @author: shudonghui
 * @date: 2023-08-07 13:53:29
 * @version: 1.0
 * @email: shudonghui@qmth.com.cn
 * @Company: www.qmth.com.cn
 */
@Service
public class SysMessageServiceImpl extends ServiceImpl<SysMessageMapper, SysMessage> implements SysMessageService {


    @Resource
    SysUserService sysUserService;


    @Resource
    TBUserArchivesSupplierService tbUserArchivesSupplierService;

    @Resource
    TBServiceService tbServiceService;

//    /**
//     * 查询列表
//     *
//     * @param iPage
//     * @return
//     */
//    @Override
//    public IPage<SysMessage> query(IPage<Map> iPage, String query) {
//        return this.baseMapper.query(iPage, query);
//    }
//
//    /**
//     * 新增修改系统消息
//     *
//     * @param sysMessage
//     */
//    @Override
//    @Transactional
//    public Boolean saveSysMessage(SysMessage sysMessage) {
//        try {
//            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
//            if (Objects.isNull(sysMessage.getId())) {// 新增
//                //sysMessage.insertInfo(sysUser.getId());
//            } else { // 修改
//                //sysMessage.updateInfo(sysUser.getId());
//            }
//            return saveOrUpdate(sysMessage);
//        } catch (Exception e) {
//            if (e instanceof DuplicateKeyException) {
//                String errorColumn = e.getCause().toString();
//                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
//                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
//            } else if (e instanceof ApiException) {
//                ResultUtil.error((ApiException) e, e.getMessage());
//            } else {
//                ResultUtil.error(e.getMessage());
//            }
//        }
//        return null;
//
//    }
//
//    /**
//     * 删除系统消息
//     *
//     * @param id
//     */
//    @Override
//    @Transactional
//    public Boolean delete(Long id) {
//        return this.removeById(id);
//    }

    @Override
    public IPage<SysMessageResult> query(IPage<Map> iPage, Long noticeId, String city, Long supplierId, Boolean status) {
        return this.baseMapper.query(iPage, noticeId,city,supplierId,status);
    }

    @Override
    public List<SysMessageResult> query(Long noticeId, String city, Long supplierId, Boolean status) {
        return this.baseMapper.query(noticeId,city,supplierId,status);
    }

    @Override
    public SysMessageCountResult count(long noticeId) {
        return this.baseMapper.count(noticeId);
    }

    @Override
    @Transactional
    public void deleteByNoticeId(long noticeId) {
        this.remove(new QueryWrapper<SysMessage>().lambda().eq(SysMessage::getNoticeId, noticeId));
    }

    @Override
    @Transactional
    public void publishByNotice(SysNotice sysNotice) {
        if (sysNotice.getType() == NoticeTypeEnum.SYSTEM) {
            List<SysUser> sysUsers = sysUserService.listEnable();
            sysUsers.forEach(s -> {
                insertMessage(sysNotice, s.getId());
            });
        } else if (sysNotice.getType() == NoticeTypeEnum.SUPPLIER) {
            List<SysUser> sysUsers = tbUserArchivesSupplierService.findUsers(sysNotice.getSupplierId());
            sysUsers.forEach(s -> {
                insertMessage(sysNotice, s.getId());
            });
        } else if (sysNotice.getType() == NoticeTypeEnum.SERVICE) {
            List<TBService> tbServices = tbServiceService.listEnable(sysNotice.getServiceId());
            tbServices.forEach(s -> {
                insertMessage(sysNotice, s.getServiceLeadId());
            });
        }

    }

    private void insertMessage(SysNotice sysNotice, Long s) {
        SysMessage sysMessage = new SysMessage();
        sysMessage.setContent(sysNotice.getContent());
        sysMessage.setCrmNo(null);
        sysMessage.setFormUserId(sysNotice.getCreateId());
        sysMessage.setNoticeId(sysNotice.getId());
        sysMessage.setReceiveUserId(s);
        sysMessage.setSendTime(System.currentTimeMillis());
        sysMessage.setServiceId(sysNotice.getServiceId());
        sysMessage.setStatus(false);
        sysMessage.setSupplierId(sysNotice.getSupplierId());
        sysMessage.setType(MessageTypeEnum.SYSTEM);
        this.save(sysMessage);
    }

}